
#include "scia_defs.h"
#include "bin_read.h"

/***********************************************************************
 * void float_getbin (FILE* unit, float *var)
 * void float_array_getbin (FILE* unit, float *var, int nr)
 *
 * read binary float (float array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void float_getbin (FILE* unit, float *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(float)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(float), 1, unit);
#endif
}


void float_array_getbin (FILE* unit, float *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int n;
    int k;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(float)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(float) + n , sizeof(char), 1, unit);
    }
#else
	size = fread (vo, sizeof(float), nr, unit);
#endif	
}
    
/***********************************************************************
 * void double_getbin (FILE* unit, double *var)
 * void double_array_getbin (FILE* unit, double *var, int nr)
 *
 * read binary double (double array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void double_getbin (FILE* unit, double *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(double)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(double), 1, unit);
#endif
}


void double_array_getbin (FILE* unit, double *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(double)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(double) + n , sizeof(char), 1, unit);
    }
#else
	size = fread (vo, sizeof(double), nr, unit);
#endif	
}
    

/***********************************************************************
 * void int_getbin (FILE* unit, int *var)
 * void int_array_getbin (FILE* unit, int *var, int nr)
 *
 * read binary int (int array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void int_getbin (FILE* unit, int *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(int)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(int), 1, unit);
#endif
}


void int_array_getbin (FILE* unit, int *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(int)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(int) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(int), nr, unit);
#endif	
}
    

/***********************************************************************
 * void uint_getbin (FILE* unit, unsigned int *var)
 * void uint_array_getbin (FILE* unit, unsigned int *var, int nr)
 *
 * read binary unsigned int (unsigned int array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/
void uint_getbin (FILE* unit, unsigned int *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned int)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned int), 1, unit);
#endif
}


void uint_array_getbin (FILE* unit, unsigned int *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned int)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned int) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned int), nr, unit);
#endif	
}
    

/***********************************************************************
 * void short_getbin (FILE* unit, short *var)
 * void short_array_getbin (FILE* unit, short *var, int nr)
 *
 * read binary short (short array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/
void short_getbin (FILE* unit, short *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(short)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(short), 1, unit);
#endif
}


void short_array_getbin (FILE* unit, short *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(short)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(short) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(short), nr, unit);
#endif	
}
    

/***********************************************************************
 * void ushort_getbin (FILE* unit, unsigned short *var)
 * void ushort_array_getbin (FILE* unit, unsigned short *var, int nr)
 *
 * read binary unsigned short (unsigned short array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void ushort_getbin (FILE* unit, unsigned short *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;

/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned short)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned short), 1, unit);
#endif
}


void ushort_array_getbin (FILE* unit, unsigned short *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned short)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned short) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned short), nr, unit);
#endif	
}
    

/***********************************************************************
 * void long_getbin (FILE* unit, long *var)
 * void long_array_getbin (FILE* unit, long *var, int nr)
 *
 * read binary long (long array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void long_getbin (FILE* unit, long *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(long)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(long), 1, unit);
#endif
}


void long_array_getbin (FILE* unit, long *var, int nr)
{ 
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(long)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(long) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(long), nr, unit);
#endif	
}
    
/***********************************************************************
 * void ulong_getbin (FILE* unit, unsigned long *var)
 * void ulong_array_getbin (FILE* unit, unsigned long *var, int nr)
 *
 * read binary unsigned long (unsigned long array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void ulong_getbin (FILE* unit, unsigned long *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned long)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned long), 1, unit);
#endif
}


void ulong_array_getbin (FILE* unit, unsigned long *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned long)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned long) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned long), nr, unit);
#endif	
}
    


/***********************************************************************
 * void char_getbin (FILE* unit, char *var)
 * void char_array_getbin (FILE* unit, char *var, int nr)
 *
 * read binary char (char array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void char_getbin (FILE* unit, char *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

    size = fread (vo, sizeof(char), 1, unit);
}


void char_array_getbin (FILE* unit, char *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    size = fread (vo, sizeof(char), nr, unit);
}
    
/***********************************************************************
 * void uchar_getbin (FILE* unit, unsigned char *var)
 * void uchar_array_getbin (FILE* unit, unsigned char *var, int nr)
 *
 * read binary unsigned char (unsigned char array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void uchar_getbin (FILE* unit, unsigned char *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

    size = fread (vo, sizeof(unsigned char), 1, unit);
}


void uchar_array_getbin (FILE* unit, unsigned char *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    size = fread (vo, sizeof(unsigned char), nr, unit);
}
    
