#ifndef  __GOME_L2                       /* Avoid multiple includes */
#define  __GOME_L2

#include "scia_defs.h"
#include "scia_common.h"
#include "bin_read.h"


#define GOME_NMOL_MAX 7
#define GOME_NWIN_MAX 3

typedef enum GOME_L2_MOL
{
    GOME_L2_MOL_O3,
    GOME_L2_MOL_O3vis,
    GOME_L2_MOL_NO2,
    GOME_L2_MAX
} GOME_L2_MOL;

extern char GOME_L2_MOL_NAME [GOME_L2_MAX][6];
extern int GOME_L2_MOL_WIN [GOME_L2_MAX];

typedef struct UTC_s 
{
	unsigned int Days;
		/* Days since 1st January 1950 */
	unsigned int Msecs;
		/* Milliseconds since midnight */
}  UTC_s ;



typedef struct MPH_GOME
{
    /* Product Identifier Record see table 3 */
    char pir[39];
    /* Number of SPH2 Records (always 1) */
    unsigned short n_sph;
    /* Length of SPH2 Record */
    int length_sph;
    /* Number of DOAS Data Records (ca. 2200) */
    unsigned short n_doas;
    /* Length of DOAS Data Record */
    int length_doas;
} MPH_GOME;

typedef struct SPH_GOME
{
    /* Input Data References see table 3 */
  char pir[39];
    /* GDP Software Version, DOAS Level 1 to 2, (XX.XX) */
  char version_DOAS[6];
    /* GDP Static Parameter File Version, (XX.XX) */
  char version_keyparam[6];
    /* GDP L2 Format Version, (XX.XX) */
  char version_format[6];
    /* Number of Fitting Windows (Nwin=3) */
  unsigned short Nwin;			// Number of Fitting Windows
    /* Window Pair (start and end wavelength in nm) */
  float windowpair[GOME_NWIN_MAX][2];
    /* Number of Molecules (Nmol=7) */
  unsigned short Nmol;			// Number of Moleculs
    /* Molecule Pair (fitting window number and molecule name, e.g. 3NO2__) */
  char molpair[GOME_NMOL_MAX][7];	// window number + molecule NO2..
    /* Atmosphere Height */
  float atmos_height;
} SPH_GOME;


typedef struct Geo_L2gome
{
    /* Ground Pixel Number */
    int gp;
    /* Subset Counter */
    int subsetcounter;
    
    /* UTC Date & Time of the ground pixel at end of integration time */
    UTC_s time;
    
    /* 3 Solar Zenith Angles at satellite for the points {A , B , C } */
    float sol_zen_angle_sat[3];
    /* 3 Line of Sight Angles at satellite for the points {A , B , C } */
    float los_zen_angle_sat[3];
    /* 3 Relative Azimuth Angles at satellite for the points {A , B , C } */
    float rel_azi_angle_sat[3];
    
    /* 3 Solar Zenith Angles at TOA for the points {A , B , C } */
    float sol_zen_angle_toa[3];
    /* 3 Line of Sight Angles at TOA for the points {A , B , C } */
    float los_zen_angle_toa[3];
    /* 3 Relative Azimuth Angles at TOA for the points {A , B , C } */
    float rel_azi_angle_toa[3];
    
    /* Satellite Geodetic Height at point {B } */
    float sat_geod_ht;
    /* Earth Radius of Curvature at point {B } */
    float earth_rad;
    /* 4 Corner and Center Coordinates (Lat., Long.) at points
       {1 , 2 , 3 , 4  and 5 } of Ground Pixel */
    Coord_deg cor_coor_nad[4];
        /*  Centre coordinate of the nadir ground pixel */
    Coord_deg cen_coor_nad;
} Geo_L2gome ;



/* GOME Intermediate Results Record Content */

typedef struct Gome_IRR
{
/* VCD: Total VCD to ground of e.g. O3 UV, O3 VIS, NO2 UV, NO2 VIS, BrO, H2O, SO2 */
  float VCD_dat[GOME_NMOL_MAX];
/* VCD: Errors on VCDs above */
  float VCD_err[GOME_NMOL_MAX];
/* VCD: Flag indexing output */
  unsigned short VCD_ind;
/* DOAS: Slant Columns of e.g. O3 UV, O3 VIS, NO2 UV, NO2 VIS, H2O, BrO, SO2 */
  float SlantCol[GOME_NMOL_MAX];
/* DOAS: Errors on Slant Columns above */
  float SlantCol_err[GOME_NMOL_MAX];
/* DOAS: RMS,  2, Goodness of Fit, Iteration Number for the Nwin fitting windows */
  float windowinf[4*GOME_NWIN_MAX];
/* DOAS: Flag indexing output */
  unsigned short DOAS_ind;
/* AMF: AMF to ground for species as listed above */
  float AMF_ground[GOME_NMOL_MAX];
/* AMF: AMF to cloud top for species as listed above */
  float AMF_top[GOME_NMOL_MAX];
/* AMF: Total intensities for ground for Nwin fitting windows */
  float AMF_winground[GOME_NWIN_MAX];
/* AMF: Total intensities for cloud top for Nwin fitting windows */
  float AMF_wintop[GOME_NWIN_MAX];
/* AMF: Measured Intensity for Nwin windows */
  float AMF_winmeasured[GOME_NWIN_MAX];
/* AMF: Flag indexing output */
  unsigned short AMF_ind;
/* ICFA: Cloud fraction and Cloud top Pressure */
/* ICFA: Errors on Cloud fraction and Cloud  top Pressure */
/* ICFA: Surface Pressure */
  float cloud[5];
/* ICFA: CCA Cloud Fraction */
  float CCA_cf;
/*  ICFA: 16 CCA Subpixel Trace */
  unsigned char CCA_subpixel[16];
/* ICFA: Flag indexing output */
  unsigned short cloud_ind;
/* Statistics: Pixel contrast (3 PMDs: mean and standard deviation) ! */
  float stat_contrast[6];
/* Statistics: Pixel colour (sub pixel values) ! */
  float stat_pixcolour[16];
/* Statistics: Pixel colour gradient (Ch. 3) ! */
  float stat_gradient;
/* Statistics: Flag indexing output */
  unsigned short stat_ind;
} Gome_IRR;


   

/* One Dataset of GOME data */

typedef struct Gome_Doas
{
/* Geolocation Record */
    GeoN_L2_user geo;   
/* Total Column of Ozone (Dobson Units) */
    float ozon;
/* Relative error on the Total Column (%) */
    float ozon_err;
/* Intermediate Results Record  */
    Gome_IRR gome_irr;
} Gome_Doas;
		       


void UTC_s_getbin (FILE* unit, UTC_s *var);
void MPH_GOME_getbin (FILE* unit, MPH_GOME *var);
void SPH_GOME_getbin (FILE* unit, SPH_GOME *var);
MJD UTC_s_2_MJD (UTC_s *utc_s);
void Geo_L2gome_getbin (FILE* unit, GeoN_L2_user *var);
void Gome_IRR_getbin (FILE* unit, Gome_IRR *var, int Nmol, int Nwin);
void Gome_Doas_getbin (FILE* unit, Gome_Doas *var, int Nmol, int Nwin);


#endif
