#ifndef  __SCIA_L2MET                       /* Avoid multiple includes */
#define  __SCIA_L2MET

#include "scia_defs.h"
#include "scia_common.h"
#include "bin_read.h"


typedef struct SPH_SCI_RV__2P {
        /*  SPH descriptor */
    char sph_descriptor[28];
        /*  Value: +000 = No stripline continuity, the product is a complete
            segment Other: Stripline Counter */
    char stripline_continuity_indicator[5];
        /*  Value: +001 to NUM_SLICES Default value if no stripline continuity
            = +001 */
    char slice_position[5];
        /*  Number of slices in this stripline Default value if no continuity
            = +001 */
    char num_slices[5];
        /*  Start time of the measurement data in this product. UTC time of
            first MDSR */
    char start_time[28];
        /*  Time of the end of the measurement data in this product (last MDSR
            time). UTC time of last MDSR */
    char stop_time[28];
        /*  Latitude of the satellite nadir at the START_TIME. WGS84 latitude,
            positive = North */
    char start_lat[12];
        /*  Longitude of the satellite nadir at the START_TIME. WGS84 longitude,
            positive = East, 0 = Greenwich */
    char start_long[12];
        /*  Latitude of the satellite nadir at the STOP_TIME. WGS84 latitude,
            positive = North */
    char stop_lat[12];
        /*  Longitude of the satellite nadir at the STOP_TIME. WGS84 longitude,
            positive = East, 0 = Greenwich */
    char stop_long[12];
        /*  Spare */
    char spare_1[50];
        /*  Quality summary of the fitting errors: GOOD or FAIR or BAD&Oslash; */
    char fitting_error_sum[4];
        /*  Wavelength range of fitting window given as a pattern of two wavelengths
            like: xxxx-yyyy e.g., 0325-0335 */
    char doas_fitting_window_0[9];
} SPH_SCI_RV__2P ;



/* Array with DSD Names : declaration, definition in scia_L2.c*/

/* DSD name enum */
    extern char DS_NAME_SCI_RV__2P [2][29];

typedef enum DSD_SCI_RV__2P
{
    SCI_RV__2P_GEOLOCATION,
    SCI_RV__2P_O3_VERTICAL_COLUMN,
    SCI_RV__2P_MAX_DS_NAME
} DSD_SCI_RV__2P;



typedef struct O3_met {
        /*  Start time of the DSR */
    MJD dsr_time;
        /*  Quality Indicator (-1 for blank MDSR, 0 otherwise) */
    char quality_flag;
        /*  Integration time for this DSR */
    unsigned short integr_time;
        /*  Ozone Vertical Column Density */
    float o3_vcd;
        /*  Error on the VCD above */
    float err_vcd;
} O3_met ;


typedef struct GeoN_L2MET {
        /*  Start time of the DSR */
    MJD dsr_time;
        /*  Attachment Flag (set to 1 if all MDSRs corresponding to this ADSR
            are blank, set to zero otherwise) */
    char attach_flag;
        /*  Integration time of this geolocation entity */
    unsigned short integr_time;
        /*  Four corner co-ordinates of the nadir ground pixels */
    Coord coor_grd[4];
} GeoN_L2MET ;



SCIA_err Read_SPH_SCI_RV__2P (FILE* unit, SPH_SCI_RV__2P *sph);

void O3_met_getbin (FILE* unit, O3_met *var);

void GeoN_L2MET_getbin (FILE* unit, GeoN_L2MET *var);

#endif
