#ifndef  __SCIA_L2N_LIB                       /* Avoid multiple includes */
#define  __SCIA_L2N_LIB

#include "scia_defs.h"

#include "scia_common.h"
#include "scia_L2N.h"
#include "scia_L2MET.h"
#include "scia_L2O.h"
#include "gome_L2.h"
#include "geo_distance.h"

/* used for mean geolocation in LIMB scan, values are taken from about 20km */

extern int limb_mean_height_idx;  /* 27 */


/* Structure for holding product ids */
typedef struct product_id {
    DSD_SCI_NL__2P prod_n;
    DSD_SCI_OL__2P prod_o;
    DSD_SCI_RV__2P prod_m;
    DSD_SCI_OL__2P prod_l;
    GOME_L2_MOL prod_g;
} product_id;

/* enum for generic product names */
typedef enum GENERIC_PROD {
    GEN_CLOUDS_AEROSOL,
    GEN_O3,
    GEN_NO2,
    GEN_O3VIS,
    GEN_BRO,
    GEN_OCLO,
    GEN_SO2,
    GEN_HCHO,
    GEN_H20,
    GEN_CO2,
    GEN_N20,
    GEN_CH4,
    GEN_CO,
    MAX_GENERIC_PROD
} GENERIC_PROD;

/* Name of generic product names */
extern char Generic_Prod [MAX_GENERIC_PROD][15];

/* table for allocation of real products to generic products */
/* has to be adjusted, when fittingwindows are fixed/known for NRT/OL */
/* definition is in scia_L2_lib.c */

extern int gen_product_matrix [MAX_GENERIC_PROD][5];


/* enum for defining type of output in user_data */
typedef enum MDS_TYPE {
    BIAS,       /* L2N BIAS */
    DOAS,	/* L2N DOAS */
    CLD_AER,	/* L2N CLOUD_AEROSL */
    METEO,	/* L2met */
    NADIR_OL,	/* L2OL (BIAS+DOAS) */
    CLD_AER_OL,  /* L2ol CLOUD_AEROSL */
    LIMB_OL,     /* L2ol LIMB profiles */
    GOME
} MDS_TYPE;


/* enum for specifiying output mode */
enum { NADIR, LIMBPROFILE, LIMBTABLE };


typedef struct GeoL_L2_user {
        /*  Start Time of DSR */
    MJD dsr_time;
        /*  Attachment Flag */
    char attach_flag;
        /*  Integration time for this geolocation entity */
    unsigned short integr_time;
        /*  Solar zenith angles of the start, middle and end of the integration
            time at TOA */
    float sol_zen_angle_toa[3];
        /*  Line-of-sight zenith angles of start, middle and end of the integration
            time at TOA */
    float los_zen_angle_toa[3];
        /*  Relative azimuth angles of start, middle and end of the integration
            time at TOA */
    float rel_azi_angle_toa[3];
        /*  Satellite geodetic height at the middle of the integration time */
    float sat_geod_ht;
        /*  Earth radius at the middle of the integration time */
    float earth_rad;
        /*  Sub-satelite point at the middle of the integration time */
    Coord_deg sub_sat_point;
        /*  Co-ordinates of tangent ground points of the start, middle and
            end of the integration time */
    Coord_deg tangent_coord[3];
        /*  Tangent height of the start, middle and end of the integration
            time */
    float tangent_height[3];
} GeoL_L2_user ;







/* Structure for holding information needed to be stored after opening the file */


typedef struct info_l2N
{
    FILE *FILE_l2N;		/* File handler for Lv1C - file */
    MPH mph;			/* Main product header */    
    MPH_GOME mph_gome;		/* Main product header GOME */
    char orbit_date[MAX_UTC_STRING]; 	/* Orbit start time / sensing start */
    
    SCIA_PRODUCT scia_product;	/* L2N L2O or L2MET? or GOME */
    
    SPH_SCI_NL__2P sph;		/* Specific Product header */
    SPH_SCI_RV__2P sph_met;		/* Specific Product header */
    SPH_SCI_OL__2P sph_ol;

    SPH_GOME sph_gome;		/* Specific Product header GOME */
    
				/* Following arrays are smaller */
				/*  for meteo: MAX_DS_NAME_L2MET */
				/*  would be enough */

    DSD dsd [ SCI_OL__2P_MAX_DS_NAME ];/* Array for DSDs (OL product has most  */
				/* Offset behind last MDS read */
    unsigned cur_offset [ SCI_OL__2P_MAX_DS_NAME ];
				/* Count reads */
    unsigned cur_n [ SCI_OL__2P_MAX_DS_NAME ];
    unsigned geo_n [ SCI_OL__2P_MAX_DS_NAME ];
    
    unsigned n_geo;
    GeoN_L2_user *geo;		/* Pointer for holding geolocation array */
    unsigned n_geo_L;
    GeoL_L2_user *geo_L;	/* Pointer for holding geolocation array Limb*/
    int limb_flag;
				/*  States information is used for unique
				    limb time stamp */
    unsigned n_states;
    States *states_L;
    unsigned states_n [SCI_OL__2P_MAX_DS_NAME ];
				/* which profile in state */
    int n_geoprofile_in_state [SCI_OL__2P_MAX_DS_NAME ];
				/* how many profiles expected per state? */
    int n_geoprofile_per_state [SCI_OL__2P_MAX_DS_NAME ];
    
				/* GOME specific part */
    int gome_nwin;		/* Number of fitting windows */
    int gome_nmol;		/* Number of molecules */
    
} info_l2N;
 


typedef struct user_data
{
    /* MPH */
    char *product ; /* Product File name ascii */
    char *software_ver ; /* Software Version number of processing software */
    int abs_orbit; /* Start absolute orbit number               */
				/*  Sensing start of orbit */
    char *orbit_date;
    
    /* type */
    MDS_TYPE method;			/* BIAS DOAS ....*/
    product_id prod;			/* Product : ID from DSD-enums!! */
    char prod_str[MAX_L2N_PROD_STR];
    char *molecule;		/*  */
    char *doas_fitting_window;
    char *bias_fitting_window;
				/*  for Nadir OL-IR, more spezies are fitted */
				/*  simultanously, here store the names */
    char prod_str_ir[N_MAX_LIMB_SPECIES][MAX_L2N_PROD_STR];
    int n_prod;
    int main_prod;		/* used for Nadir OL-IR and select GOME sp. */
    int main_win;		/* store number of fitwin for GOME */
    /* geolocation */
    GeoN_L2_user geo;		/* Geolocation, common for all Nadir types */
    GeoL_L2_user geo_L;		/* Geolocation for limb  */
    Coord_deg corner_limb[4];	/* "Corner" coordinates for limb */
    int subsetcounter;		/* identifier for profile number in state */
    
    /* only part of the measured limb scans is used in retrieval */
    int n_geo_0;		/* Start of used limb geolocation */
    int n_geo_L;		/* number of used limb geolocateion */
				/* Detailed Geolocation for limb
				   (complete scan)  */
    int n_profile_in_state;	/* number of this profile in state  */
    GeoL_L2_user geo_L_array [LIMB_LEVELS];
				/*  Time as string and frac day */
    char date_str [MAX_UTC_STRING];
    double frac_day;
				/* start time of the state,
				   used only in limb for unique time stamp */
    char state_date_str[MAX_UTC_STRING];
    int n_state;		/* actual index of state */
    States *state;		/* Pointer to actual state (limb only) */
    /* data */
    DOAS_L2N doas;
    BIAS_L2N bias;
    CLOUDS_AEROSOL_L2N cld_aer;
    O3_met met;
    Nadir_OL nad_ol;
    Clouds_Aerosol_L2O cld_aer_ol;
    Limb_OL lim_ol;
    Gome_Doas gome;
} user_data;

/*
typedef struct user_bias
{
    DSD_L2N prod;
    char *bias_molecule;
    GeoN_L2 geo;
    BIAS_L2N bias;
} user_bias;
*/

SCIA_err openL2N (char* FILE_name, info_l2N *info);

SCIA_err closeL2N (info_l2N *info);

int gome_check (FILE* file);


SCIA_err GeoN_L2_add (GeoN_L2_user *geo_coadd, int *n_coadd, info_l2N *info,
		      int prod, int integr_time);
Coord_deg middle_coord (Coord_deg p1, Coord_deg p2);


SCIA_err L2N_next_data (info_l2N *info, product_id prod,
			 user_data *ud);

product_id prod_id_str (const char* prod_str);

void prod_id_print (FILE *out);

void GeoN_L2_user_array_getbin (FILE* unit, GeoN_L2_user *var, int nr);

void GeoN_L2_user_met_array_getbin (FILE* unit, GeoN_L2_user *var, int nr);

void GeoN_L2_OL_user_array_getbin (FILE* unit, GeoN_L2_user *var, int nr);

void GeoL_L2_OL_user_array_getbin (FILE* unit, GeoL_L2_user *var, int nr);

SCIA_err L2N_info (FILE* out, info_l2N* info);

SCIA_err write_OL_initialisation_file (char *xml_dir, info_l2N *info);

double distance (user_data *ud, Coord_deg p);

double distance_limb (user_data *ud, Coord_deg p);

SCIA_err Geo_Limb_mean (GeoL_L2_user *geo_arr,
			GeoL_L2_user *geo);

SCIA_err Geo_limb_coadd (info_l2N *info,
			 GeoL_L2_user *geo_coadd,
			 GeoL_L2_user *geo_L,
			 int n_coadd);

SCIA_err Geo_limb_swap ( GeoL_L2_user *geo_L);

#endif /* End SCIA_L2N_LIB */
